@@ -12,18 +12,19 @@ from .basic_manifold_jax import basic_manifold_jax
 
 
 class euclidean_jax(basic_manifold_jax):
-    def __init__(self, var_shape, train_mode = False) -> None:
+    def __init__(self, var_shape, train_mode = False, device = None) -> None:
 
         self.dim = jnp.prod(jnp.array(var_shape))
         self.var_shape = var_shape
         self.manifold_type = 'S'
+        self.device = device
 
         
         
         self.zero_scalar = 0
         if not train_mode:
             self.zeroX = jnp.zeros(self.var_shape)
-            super().__init__('eclidean',var_shape, (1,))
+            super().__init__('eclidean',var_shape, (1,), device= self.device)
 
         
 
@@ -75,7 +76,7 @@ class euclidean_jax(basic_manifold_jax):
         if Xinit is None:
             key = random.PRNGKey(seed)
             Xinit = random.normal(key, self.var_shape)
-            
+        Xinit = jax.device_put(Xinit, self.device)
         return Xinit
 
     def Post_process(self,X):
