@@ -11,7 +11,8 @@ from torch.nn.utils.rnn import PackedSequence
 from torch.nn import init
 from torch import _VF
 from ...manifold_torch import euclidean_torch
-
+from ..utils.modified_apply import modified_apply
+from .utils import wvt_identical
 
 _rnn_impls = {
     'RNN_TANH': _VF.rnn_tanh,
@@ -41,7 +42,7 @@ class RNNBase_cdopt(Module):
     def __init__(self, mode: str, input_size: int, hidden_size: int,
                  num_layers: int = 1, bias: bool = True, batch_first: bool = False,
                  dropout: float = 0., bidirectional: bool = False, proj_size: int = 0,
-                 device=None, dtype=None, manifold_class = euclidean_torch, penalty_param = 0, weight_var_transfer = None, manifold_args = {}) -> None:
+                 device=None, dtype=None, manifold_class = euclidean_torch, penalty_param = 0, weight_var_transfer = wvt_identical, manifold_args = {}) -> None:
         factory_kwargs = {'device': device, 'dtype': dtype}
         super(RNNBase_cdopt, self).__init__()
         self.mode = mode
@@ -135,13 +136,7 @@ class RNNBase_cdopt(Module):
 
         
 
-        if weight_var_transfer is None:
-            def weight_var_transfer(tensor_shape):
-                weight_to_var = lambda X_tensor: X_tensor
-                var_to_weight = lambda X_var: X_var 
-                var_shape = tensor_shape 
-
-                return weight_to_var, var_to_weight, var_shape 
+        
 
         self.weight_to_var, self.var_to_weight, self.var_shape = weight_var_transfer( (self.hidden_size, self.hidden_size) )
 
@@ -165,7 +160,7 @@ class RNNBase_cdopt(Module):
         self.flatten_parameters()
 
         for key, param in self.manifold._parameters.items():
-            self._parameters[key] = param
+            self.register_buffer(key, param, persistent=False)
 
         
 
@@ -318,7 +313,8 @@ class RNNBase_cdopt(Module):
         return A_flat_weight
 
     def _apply(self, fn):
-        ret = super(RNNBase_cdopt, self)._apply(fn)
+        # ret = super(RNNBase_cdopt, self)._apply(fn)
+        ret = modified_apply(self, fn)
 
         # Resets _flat_weights
         # Note: be v. careful before removing this, as 3rd party device types
@@ -1038,7 +1034,7 @@ class RNNCellBase_cdopt(Module):
     # See https://github.com/pytorch/pytorch/issues/39670
 
     def __init__(self, input_size: int, hidden_size: int, bias: bool, num_chunks: int,
-                 device=None, dtype=None, manifold_class = euclidean_torch, penalty_param = 0, weight_var_transfer = None, manifold_args = {}) -> None:
+                 device=None, dtype=None, manifold_class = euclidean_torch, penalty_param = 0, weight_var_transfer = wvt_identical, manifold_args = {}) -> None:
         factory_kwargs = {'device': device, 'dtype': dtype}
         super(RNNCellBase_cdopt, self).__init__()
         self.input_size = input_size
@@ -1055,13 +1051,7 @@ class RNNCellBase_cdopt(Module):
 
 
 
-        if weight_var_transfer is None:
-            def weight_var_transfer(tensor_shape):
-                weight_to_var = lambda X_tensor: X_tensor
-                var_to_weight = lambda X_var: X_var 
-                var_shape = tensor_shape 
 
-                return weight_to_var, var_to_weight, var_shape 
 
         self.weight_to_var, self.var_to_weight, self.var_shape = weight_var_transfer( ( num_chunks * self.hidden_size, self.hidden_size) )
 
@@ -1080,7 +1070,7 @@ class RNNCellBase_cdopt(Module):
         self.reset_parameters()
         
         for key, param in self.manifold._parameters.items():
-            self._parameters[key] = param
+            self.register_buffer(key, param, persistent=False)
 
         
 
@@ -1099,6 +1089,9 @@ class RNNCellBase_cdopt(Module):
 
         self.weight_hh = Parameter(self.manifold.Init_point(self.weight_hh))
 
+    def _apply(self, fn):
+        return modified_apply(self, fn)
+
 
 
 
